@@ -19,13 +19,18 @@ from oslotest import base
 from six.moves import builtins
 
 from os_win import exceptions
+from os_win.utils import baseutils
+
+
+class TestingException(Exception):
+    pass
 
 
 class FakeWMIExc(exceptions.x_wmi):
     def __init__(self, hresult=None):
+        super(FakeWMIExc, self).__init__()
         excepinfo = [None] * 5 + [hresult]
         self.com_error = mock.Mock(excepinfo=excepinfo)
-        super(FakeWMIExc, self).__init__()
 
 
 class OsWinBaseTestCase(base.BaseTestCase):
@@ -33,6 +38,7 @@ class OsWinBaseTestCase(base.BaseTestCase):
         super(OsWinBaseTestCase, self).setUp()
 
         self._mock_wmi = mock.MagicMock()
+        baseutils.BaseUtilsVirt._old_wmi = self._mock_wmi
 
         mock_os = mock.MagicMock(Version='6.3.0')
         self._mock_wmi.WMI.return_value.Win32_OperatingSystem.return_value = (
