@@ -359,48 +359,6 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         mock_modify_virtual_system.assert_called_once_with(
             mock_vmsettings)
 
-    @mock.patch.object(vmutils.VMUtils, '_set_vm_memory')
-    @mock.patch.object(vmutils.VMUtils, '_create_vm_obj')
-    def test_vnuma_create_vm(self, mock_create_vm_obj, mock_set_mem):
-        self._vmutils._vs_man_svc.DefineSystem.return_value = (
-            None, self._FAKE_JOB_PATH, self._FAKE_RET_VAL)
-        self._vmutils.create_vm(self._FAKE_VM_NAME,
-                                mock.sentinel.vnuma_enabled,
-                                self._VM_GEN,
-                                mock.sentinel.instance_path)
-
-        mock_create_vm_obj.assert_called_once_with(
-            self._FAKE_VM_NAME, mock.sentinel.vnuma_enabled,
-            self._VM_GEN, None, mock.sentinel.instance_path)
-        self.assertFalse(mock_set_mem.called)
-
-    @mock.patch.object(vmutils.VMUtils, '_set_vm_vcpus')
-    @mock.patch.object(vmutils.VMUtils, '_set_vm_memory')
-    def test_old_create_vm(self, mock_set_mem, mock_set_vcpus):
-        mock_vmsetting = self._lookup_vm()
-        mock_svc = self._vmutils._vs_man_svc
-        getattr(mock_svc, self._DEFINE_SYSTEM).return_value = (
-            None, self._FAKE_JOB_PATH, self._FAKE_RET_VAL)
-
-        self._vmutils.create_vm(self._FAKE_VM_NAME, self._FAKE_MEMORY_MB,
-                                self._FAKE_VCPUS_NUM, False,
-                                self._FAKE_DYNAMIC_MEMORY_RATIO,
-                                self._VM_GEN,
-                                mock.sentinel.instance_path)
-
-        self.assertTrue(getattr(mock_svc, self._DEFINE_SYSTEM).called)
-        mock_set_mem.assert_called_once_with(
-            mock_vmsetting,
-            self._FAKE_MEMORY_MB,
-            None,
-            self._FAKE_DYNAMIC_MEMORY_RATIO)
-
-        mock_set_vcpus.assert_called_once_with(
-            mock_vmsetting,
-            self._FAKE_VCPUS_NUM,
-            None,
-            False)
-
     @mock.patch.object(_wqlutils, 'get_element_associated_class')
     def test_get_vm_scsi_controller(self, mock_get_element_associated_class):
         self._prepare_get_vm_controller(self._vmutils._SCSI_CTRL_RES_SUB_TYPE,
@@ -464,6 +422,41 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         mock_rasds.Address = self._FAKE_ADDRESS
         mock_get_element_associated_class.return_value = [mock_rasds]
 
+    @mock.patch.object(vmutils.VMUtils, '_get_wmi_obj')
+    def test_get_ide_ctrl_addr(self, mock_get_wmi_obj):
+        mock_rasds = mock.Mock()
+        mock_rasds.ResourceSubType = self._vmutils._IDE_CTRL_RES_SUB_TYPE
+        mock_rasds.Address = mock.sentinel.ctrl_addr
+        mock_get_wmi_obj.return_value = mock_rasds
+
+        ret_val = self._vmutils._get_disk_ctrl_addr(mock.sentinel.ctrl_path)
+        self.assertEqual(mock.sentinel.ctrl_addr, ret_val)
+
+        mock_get_wmi_obj.assert_called_once_with(mock.sentinel.ctrl_path)
+
+    @mock.patch.object(vmutils.VMUtils, '_get_vm_disk_controllers')
+    @mock.patch.object(vmutils.VMUtils, '_get_wmi_obj')
+    def test_get_scsi_ctrl_addr(self, mock_get_wmi_obj, mock_get_ctrls):
+        mock_rasds = mock.Mock()
+        mock_rasds.ResourceSubType = self._vmutils._SCSI_CTRL_RES_SUB_TYPE
+        mock_rasds.associators.return_value = [mock.sentinel.vmsettings]
+        mock_get_wmi_obj.return_value = mock_rasds
+
+        mock_scsi_ctrls = ['someCtrl', self._FAKE_CTRL_PATH.upper(),
+                           'someOtherCtrl']
+        exp_ctrl_addr = 1
+
+        mock_scsi_ctrl = mock.Mock()
+        mock_scsi_ctrl.path_.side_effect = mock_scsi_ctrls
+        mock_get_ctrls.return_value = [mock_scsi_ctrl] * len(mock_scsi_ctrls)
+
+        ret_val = self._vmutils._get_disk_ctrl_addr(self._FAKE_CTRL_PATH)
+        self.assertEqual(exp_ctrl_addr, ret_val)
+
+        mock_get_wmi_obj.assert_called_once_with(self._FAKE_CTRL_PATH)
+        mock_get_ctrls.assert_called_once_with(
+            mock.sentinel.vmsettings, self._vmutils._SCSI_CTRL_RES_SUB_TYPE)
+
     @mock.patch.object(vmutils.VMUtils, 'get_free_controller_slot')
     @mock.patch.object(vmutils.VMUtils, '_get_vm_scsi_controller')
     def test_attach_scsi_drive(self, mock_get_vm_scsi_controller,
@@ -582,11 +575,13 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         self.assertRaises(exceptions.DiskNotFound,
                           self._vmutils.get_disk_attachment_info,
                           mock.sentinel.disk_path,
-                          mock.sentinel.is_physical)
+                          mock.sentinel.is_physical,
+                          mock.sentinel.serial)
 
         mock_get_disk_res.assert_called_once_with(
             mock.sentinel.disk_path,
-            mock.sentinel.is_physical)
+            mock.sentinel.is_physical,
+            serial=mock.sentinel.serial)
 
     @ddt.data(True, False)
     @mock.patch.object(vmutils.VMUtils,
@@ -595,7 +590,9 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
                        '_get_disk_controller_type')
     @mock.patch.object(vmutils.VMUtils,
                        '_get_wmi_obj')
+    @mock.patch.object(vmutils.VMUtils, '_get_disk_ctrl_addr')
     def test_get_disk_attachment_info(self, is_physical,
+                                      mock_get_disk_ctrl_addr,
                                       mock_get_wmi_obj,
                                       mock_get_disk_ctrl_type,
                                       mock_get_disk_res):
@@ -608,7 +605,8 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         exp_att_info = dict(
             controller_slot=fake_slot,
             controller_path=exp_res.Parent,
-            controller_type=mock_get_disk_ctrl_type.return_value)
+            controller_type=mock_get_disk_ctrl_type.return_value,
+            controller_addr=mock_get_disk_ctrl_addr.return_value)
 
         att_info = self._vmutils.get_disk_attachment_info(
             mock.sentinel.disk_path,
@@ -618,6 +616,7 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         if not is_physical:
             mock_get_wmi_obj.assert_called_once_with(mock_res.Parent)
         mock_get_disk_ctrl_type.assert_called_once_with(exp_res.Parent)
+        mock_get_disk_ctrl_addr.assert_called_once_with(exp_res.Parent)
 
     @ddt.data(vmutils.VMUtils._SCSI_CTRL_RES_SUB_TYPE,
               vmutils.VMUtils._IDE_CTRL_RES_SUB_TYPE)
@@ -683,10 +682,31 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         self._vmutils._jobutils.add_virt_resource.assert_called_once_with(
             mock_nic, mock_vm)
 
+    def test_get_nic_data_by_name(self):
+        nic_cls = self._vmutils._conn.Msvm_SyntheticEthernetPortSettingData
+        nic_cls.return_value = [mock.sentinel.nic]
+
+        nic = self._vmutils._get_nic_data_by_name(mock.sentinel.name)
+
+        self.assertEqual(mock.sentinel.nic, nic)
+        nic_cls.assert_called_once_with(ElementName=mock.sentinel.name)
+
+    def test_get_missing_nic_data_by_name(self):
+        nic_cls = self._vmutils._conn.Msvm_SyntheticEthernetPortSettingData
+        nic_cls.return_value = []
+        self.assertRaises(
+            exceptions.HyperVvNicNotFound,
+            self._vmutils._get_nic_data_by_name,
+            mock.sentinel.name)
+
     @mock.patch.object(vmutils.VMUtils, '_get_nic_data_by_name')
     def test_destroy_nic(self, mock_get_nic_data_by_name):
         mock_nic_data = mock_get_nic_data_by_name.return_value
 
+        # We expect this exception to be ignored.
+        self._vmutils._jobutils.remove_virt_resource.side_effect = (
+            exceptions.NotFound(message='fake_exc'))
+
         self._vmutils.destroy_nic(self._FAKE_VM_NAME,
                                   mock.sentinel.FAKE_NIC_NAME)
 
@@ -747,28 +767,6 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         self.assertEqual(mock_diskdrive.HostResource,
                          [self._FAKE_MOUNTED_DISK_PATH])
 
-    def test_set_disk_host_resource(self):
-        self._lookup_vm()
-        mock_rasds = self._create_mock_disks()
-
-        self._vmutils._get_vm_disks = mock.MagicMock(
-            return_value=([mock_rasds[0]], [mock_rasds[1]]))
-        self._vmutils._get_disk_resource_address = mock.MagicMock(
-            return_value=self._FAKE_ADDRESS)
-
-        self._vmutils.set_disk_host_resource(
-            self._FAKE_VM_NAME,
-            self._FAKE_CTRL_PATH,
-            self._FAKE_ADDRESS,
-            mock.sentinel.fake_new_mounted_disk_path)
-        self._vmutils._get_disk_resource_address.assert_called_with(
-            mock_rasds[0])
-        self._vmutils._jobutils.modify_virt_resource.assert_called_once_with(
-            mock_rasds[0])
-        self.assertEqual(
-            mock.sentinel.fake_new_mounted_disk_path,
-            mock_rasds[0].HostResource[0])
-
     @mock.patch.object(vmutils.VMUtils, '_modify_virtual_system')
     @ddt.data(None, mock.sentinel.snap_name)
     def test_take_vm_snapshot(self, snap_name, mock_modify_virtual_system):
@@ -919,36 +917,6 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
 
         self.assertEqual(['active_vm'], active_instances)
 
-    @mock.patch.object(_wqlutils, 'get_element_associated_class')
-    def _test_get_vm_serial_port_connection(self,
-                                            mock_get_element_associated_class,
-                                            new_connection=None):
-        old_serial_connection = 'old_serial_connection'
-
-        mock_vmsettings = [self._lookup_vm()]
-
-        fake_serial_port = mock.MagicMock()
-
-        fake_serial_port.ResourceSubType = (
-            self._vmutils._SERIAL_PORT_RES_SUB_TYPE)
-        fake_serial_port.Connection = [old_serial_connection]
-        mock_rasds = [fake_serial_port]
-        mock_get_element_associated_class.return_value = mock_rasds
-        fake_modify = self._vmutils._jobutils.modify_virt_resource
-
-        ret_val = self._vmutils.get_vm_serial_port_connection(
-            self._FAKE_VM_NAME, update_connection=new_connection)
-
-        mock_get_element_associated_class.assert_called_once_with(
-            self._vmutils._conn, self._vmutils._SERIAL_PORT_SETTING_DATA_CLASS,
-            element_instance_id=mock_vmsettings[0].InstanceID)
-
-        if new_connection:
-            self.assertEqual(new_connection, ret_val)
-            fake_modify.assert_called_once_with(fake_serial_port)
-        else:
-            self.assertEqual(old_serial_connection, ret_val)
-
     @mock.patch.object(_wqlutils, 'get_element_associated_class')
     def test_get_vm_serial_ports(self, mock_get_element_associated_class):
         mock_vmsettings = self._lookup_vm()
@@ -1002,13 +970,6 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
 
         self.assertEqual(expected_ret_val, ret_val)
 
-    def test_set_vm_serial_port_connection(self):
-        self._test_get_vm_serial_port_connection(
-            new_connection='new_serial_connection')
-
-    def test_get_vm_serial_port_connection(self):
-        self._test_get_vm_serial_port_connection()
-
     def test_list_instance_notes(self):
         vs = mock.MagicMock()
         attrs = {'ElementName': 'fake_name',
@@ -1040,8 +1001,9 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         self._vmutils._jobutils.check_ret_val.assert_called_once_with(
             fake_ret_val, fake_job_path)
 
+    @ddt.data(True, False)
     @mock.patch.object(vmutils.VMUtils, '_get_wmi_obj')
-    def _test_create_vm_obj(self, mock_get_wmi_obj, vnuma_enabled=True):
+    def test_create_vm(self, mock_get_wmi_obj, vnuma_enabled=True):
         mock_vs_man_svc = self._vmutils._vs_man_svc
         mock_vs_data = mock.MagicMock()
         fake_job_path = 'fake job path'
@@ -1055,11 +1017,11 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
                                                      mock.sentinel.vm_path,
                                                      fake_ret_val)
 
-        self._vmutils._create_vm_obj(vm_name=fake_vm_name,
-                                     vm_gen=constants.VM_GEN_2,
-                                     notes='fake notes',
-                                     vnuma_enabled=vnuma_enabled,
-                                     instance_path=mock.sentinel.instance_path)
+        self._vmutils.create_vm(vm_name=fake_vm_name,
+                                vm_gen=constants.VM_GEN_2,
+                                notes='fake notes',
+                                vnuma_enabled=vnuma_enabled,
+                                instance_path=mock.sentinel.instance_path)
 
         _conn.new.assert_called_once_with()
         self.assertEqual(mock_vs_data.ElementName, fake_vm_name)
@@ -1087,12 +1049,6 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         self.assertEqual(mock.sentinel.instance_path,
                          mock_vs_data.SwapFileDataRoot)
 
-    def test_create_vm_obj(self):
-        self._test_create_vm_obj()
-
-    def test_create_vm_obj_vnuma_disabled(self):
-        self._test_create_vm_obj(vnuma_enabled=False)
-
     def test_list_instances(self):
         vs = mock.MagicMock()
         attrs = {'ElementName': 'fake_name'}
@@ -1273,64 +1229,44 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
                 vs_data, mock.sentinel.certificate_required)
             mock_modify_virtual_system.assert_called_once_with(vs_data)
 
-    @mock.patch.object(vmutils.VMUtils,
-                       '_get_mounted_disk_resource_from_path')
-    def test_set_disk_qos_specs(self, mock_get_disk_resource):
-        mock_disk = mock_get_disk_resource.return_value
+    def test_set_disk_qos_specs_exc(self):
+        self.assertRaises(exceptions.UnsupportedOperation,
+                          self._vmutils.set_disk_qos_specs,
+                          mock.sentinel.disk_path, mock.sentinel.max_iops)
 
-        self._vmutils.set_disk_qos_specs(mock.sentinel.disk_path,
-                                         max_iops=mock.sentinel.max_iops,
-                                         min_iops=mock.sentinel.min_iops)
+    def test_set_disk_qos_specs_noop(self):
+        self._vmutils.set_disk_qos_specs(mock.sentinel.disk_path, 0, 0)
 
-        mock_get_disk_resource.assert_called_once_with(
-            mock.sentinel.disk_path, is_physical=False)
-        self.assertEqual(mock.sentinel.max_iops, mock_disk.IOPSLimit)
-        self.assertEqual(mock.sentinel.min_iops, mock_disk.IOPSReservation)
-        self._vmutils._jobutils.modify_virt_resource.assert_called_once_with(
-            mock_disk)
-
-    @mock.patch.object(vmutils.VMUtils,
-                       '_get_mounted_disk_resource_from_path')
-    def test_set_disk_qos_specs_missing_values(self, mock_get_disk_resource):
-        self._vmutils.set_disk_qos_specs(mock.sentinel.disk_path)
-
-        self.assertFalse(mock_get_disk_resource.called)
-
-    def _test_is_drive_physical(self, is_physical):
-        self._vmutils._pathutils.exists.return_value = not is_physical
-        ret = self._vmutils._is_drive_physical(mock.sentinel.fake_drive_path)
-
-        self.assertEqual(is_physical, ret)
-
-    def test_is_drive_phyisical_true(self):
-        self._test_is_drive_physical(is_physical=True)
-
-    def test_is_drive_physical_false(self):
-        self._test_is_drive_physical(is_physical=False)
-
-    @mock.patch.object(vmutils.VMUtils, '_is_drive_physical')
+    @ddt.data(
+        {'drive_path':
+            r'\\ADCONTROLLER\root\virtualization\v2:Msvm_DiskDrive.'
+            r'CreationClassName="Msvm_DiskDrive",DeviceID="Microsoft:'
+            r'6344C73D-6FD6-4A74-8BE8-8EEAC2737369\\0\\0\\D",'
+            r'SystemCreationClassName="Msvm_ComputerSystem"',
+         'exp_phys_disk': True},
+        {'drive_path': 'some_image.vhdx',
+         'exp_phys_disk': False})
+    @ddt.unpack
     @mock.patch.object(vmutils.VMUtils,
                        '_get_mounted_disk_resource_from_path')
     def test_drive_to_boot_source(self, mock_get_disk_res_from_path,
-                                  mock_is_drive_physical):
-        mock_is_drive_physical.return_value = True
+                                  drive_path, exp_phys_disk):
         mock_drive = mock.MagicMock()
         mock_drive.Parent = mock.sentinel.bssd
         mock_get_disk_res_from_path.return_value = mock_drive
 
-        mock_rasd_path = mock_drive.path_.return_value
+        exp_rasd_path = (mock_drive.path_.return_value
+                         if exp_phys_disk else mock_drive.Parent)
         mock_same_element = mock.MagicMock()
         self._vmutils._conn.Msvm_LogicalIdentity.return_value = [
             mock.Mock(SameElement=mock_same_element)]
 
-        ret = self._vmutils._drive_to_boot_source(mock.sentinel.drive_path)
+        ret = self._vmutils._drive_to_boot_source(drive_path)
 
         self._vmutils._conn.Msvm_LogicalIdentity.assert_called_once_with(
-            SystemElement=mock_rasd_path)
-        mock_is_drive_physical.assert_called_once_with(
-            mock.sentinel.drive_path)
+            SystemElement=exp_rasd_path)
         mock_get_disk_res_from_path.assert_called_once_with(
-            mock.sentinel.drive_path, is_physical=True)
+            drive_path, is_physical=exp_phys_disk)
         expected_path = mock_same_element.path_.return_value
         self.assertEqual(expected_path, ret)
 
@@ -1564,3 +1500,35 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
 
     def test_remove_all_pci_devices(self):
         self._vmutils.remove_all_pci_devices(mock.sentinel.vm_name)
+
+
+class VMUtils6_3TestCase(test_base.OsWinBaseTestCase):
+
+    def setUp(self):
+        super(VMUtils6_3TestCase, self).setUp()
+        self._vmutils = vmutils.VMUtils6_3()
+        self._vmutils._conn_attr = mock.MagicMock()
+        self._vmutils._jobutils = mock.MagicMock()
+
+    @mock.patch.object(vmutils.VMUtils,
+                       '_get_mounted_disk_resource_from_path')
+    def test_set_disk_qos_specs(self, mock_get_disk_resource):
+        mock_disk = mock_get_disk_resource.return_value
+
+        self._vmutils.set_disk_qos_specs(mock.sentinel.disk_path,
+                                         max_iops=mock.sentinel.max_iops,
+                                         min_iops=mock.sentinel.min_iops)
+
+        mock_get_disk_resource.assert_called_once_with(
+            mock.sentinel.disk_path, is_physical=False)
+        self.assertEqual(mock.sentinel.max_iops, mock_disk.IOPSLimit)
+        self.assertEqual(mock.sentinel.min_iops, mock_disk.IOPSReservation)
+        self._vmutils._jobutils.modify_virt_resource.assert_called_once_with(
+            mock_disk)
+
+    @mock.patch.object(vmutils.VMUtils,
+                       '_get_mounted_disk_resource_from_path')
+    def test_set_disk_qos_specs_missing_values(self, mock_get_disk_resource):
+        self._vmutils.set_disk_qos_specs(mock.sentinel.disk_path)
+
+        self.assertFalse(mock_get_disk_resource.called)
