@@ -60,10 +60,20 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
         self.netutils._conn_attr = mock.MagicMock()
         self.netutils._jobutils = mock.MagicMock()
 
+    def test_init_caches_disabled(self):
+        self.netutils._enable_cache = False
+        self.netutils._switches = {}
+        self.netutils.init_caches()
+
+        self.netutils._conn.Msvm_VirtualEthernetSwitch.assert_not_called()
+        self.assertEqual({}, self.netutils._switches)
+
     def test_init_caches(self):
         self.netutils._switches = {}
         self.netutils._switch_ports = {}
         self.netutils._vlan_sds = {}
+        self.netutils._profile_sds = {}
+        self.netutils._hw_offload_sds = {}
         self.netutils._vsid_sds = {}
         self.netutils._bandwidth_sds = {}
         conn = self.netutils._conn
@@ -77,12 +87,16 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
 
         mock_sd = mock.MagicMock(InstanceID=self._FAKE_INSTANCE_ID)
         mock_bad_sd = mock.MagicMock(InstanceID=self._FAKE_BAD_INSTANCE_ID)
+        conn.Msvm_EthernetSwitchPortProfileSettingData.return_value = [
+            mock_bad_sd, mock_sd]
         conn.Msvm_EthernetSwitchPortVlanSettingData.return_value = [
             mock_bad_sd, mock_sd]
         conn.Msvm_EthernetSwitchPortSecuritySettingData.return_value = [
             mock_bad_sd, mock_sd]
         conn.Msvm_EthernetSwitchPortBandwidthSettingData.return_value = [
             mock_bad_sd, mock_sd]
+        conn.Msvm_EthernetSwitchPortOffloadSettingData.return_value = [
+            mock_bad_sd, mock_sd]
 
         self.netutils.init_caches()
 
@@ -90,12 +104,25 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
                          self.netutils._switches)
         self.assertEqual({mock.sentinel.port_name: mock_port},
                          self.netutils._switch_ports)
+        self.assertEqual([mock_sd], list(self.netutils._profile_sds.values()))
         self.assertEqual([mock_sd], list(self.netutils._vlan_sds.values()))
         self.assertEqual([mock_sd], list(self.netutils._vsid_sds.values()))
         self.assertEqual([mock_sd],
                          list(self.netutils._bandwidth_sds.values()))
+        self.assertEqual([mock_sd],
+                         list(self.netutils._hw_offload_sds.values()))
+
+    def test_update_cache_disabled(self):
+        self.netutils._enable_cache = False
+        self.netutils._switch_ports = {}
+        self.netutils.update_cache()
+
+        conn = self.netutils._conn
+        conn.Msvm_EthernetPortAllocationSettingData.assert_not_called()
+        self.assertEqual({}, self.netutils._switch_ports)
 
     def test_update_cache(self):
+        self.netutils._switch_ports[mock.sentinel.other] = mock.sentinel.port
         conn = self.netutils._conn
         mock_port = mock.MagicMock(ElementName=mock.sentinel.port_name)
         conn.Msvm_EthernetPortAllocationSettingData.return_value = [
@@ -106,6 +133,11 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
         self.assertEqual({mock.sentinel.port_name: mock_port},
                          self.netutils._switch_ports)
 
+        # assert that other networkutils have the same cache.
+        netutils = networkutils.NetworkUtils()
+        self.assertEqual({mock.sentinel.port_name: mock_port},
+                         netutils._switch_ports)
+
     def test_clear_port_sg_acls_cache(self):
         self.netutils._sg_acl_sds[mock.sentinel.port_id] = [mock.sentinel.acl]
         self.netutils.clear_port_sg_acls_cache(mock.sentinel.port_id)
@@ -152,6 +184,22 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
             mock.sentinel.switch_port_name)
         self.assertEqual(mock.sentinel.mac_address, actual_mac_address)
 
+    @ddt.data([], [mock.sentinel.nic_sd])
+    def test_get_vnic_settings(self, nic_sds):
+        mock_nic_sd = self.netutils._conn.Msvm_SyntheticEthernetPortSettingData
+        mock_nic_sd.return_value = nic_sds
+
+        if not nic_sds:
+            self.assertRaises(exceptions.HyperVvNicNotFound,
+                              self.netutils._get_vnic_settings,
+                              mock.sentinel.vnic_name)
+        else:
+            nic_sd = self.netutils._get_vnic_settings(mock.sentinel.vnic_name)
+            self.assertEqual(mock.sentinel.nic_sd, nic_sd)
+
+        mock_nic_sd.assert_called_once_with(
+            ElementName=mock.sentinel.vnic_name)
+
     @mock.patch.object(networkutils, 'patcher')
     @mock.patch.object(networkutils.tpool, 'execute')
     @mock.patch.object(networkutils.NetworkUtils, '_get_event_wql_query')
@@ -275,22 +323,123 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
         self.assertNotIn(self._FAKE_PORT_NAME, self.netutils._switch_ports)
         self.assertNotIn(mock_sw_port.InstanceID, self.netutils._vlan_sds)
 
-    def test_get_vswitch(self):
+    @ddt.data(True, False)
+    def test_get_vswitch(self, enable_cache):
+        self.netutils._enable_cache = enable_cache
+        self.netutils._switches = {}
         self.netutils._conn.Msvm_VirtualEthernetSwitch.return_value = [
             self._FAKE_VSWITCH]
         vswitch = self.netutils._get_vswitch(self._FAKE_VSWITCH_NAME)
 
-        self.assertEqual({self._FAKE_VSWITCH_NAME: self._FAKE_VSWITCH},
-                         self.netutils._switches)
+        expected_cache = ({self._FAKE_VSWITCH_NAME: self._FAKE_VSWITCH} if
+                          enable_cache else {})
+        self.assertEqual(expected_cache, self.netutils._switches)
         self.assertEqual(self._FAKE_VSWITCH, vswitch)
 
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_vswitch')
+    def test_get_vswitch_extensions(self, mock_get_vswitch):
+        mock_vswitch = mock_get_vswitch.return_value
+        mock_ext = mock.Mock()
+        ext_cls = self.netutils._conn.Msvm_EthernetSwitchExtension
+        ext_cls.return_value = [mock_ext] * 2
+
+        extensions = self.netutils.get_vswitch_extensions(
+            mock.sentinel.vswitch_name)
+        exp_extensions = [
+            {'name': mock_ext.ElementName,
+             'version': mock_ext.Version,
+             'vendor': mock_ext.Vendor,
+             'description': mock_ext.Description,
+             'enabled_state': mock_ext.EnabledState,
+             'extension_type': mock_ext.ExtensionType}] * 2
+
+        self.assertEqual(exp_extensions, extensions)
+
+        mock_get_vswitch.assert_called_once_with(
+            mock.sentinel.vswitch_name)
+        ext_cls.assert_called_once_with(
+            SystemName=mock_vswitch.Name)
+
+    def test_get_vswitch_cache(self):
+        self.netutils._switches = {
+            self._FAKE_VSWITCH_NAME: mock.sentinel.vswitch}
+
+        vswitch = self.netutils._get_vswitch(self._FAKE_VSWITCH_NAME)
+        self.assertEqual(mock.sentinel.vswitch, vswitch)
+
     def test_get_vswitch_not_found(self):
         self.netutils._switches = {}
         self.netutils._conn.Msvm_VirtualEthernetSwitch.return_value = []
-        self.assertRaises(exceptions.HyperVException,
+        self.assertRaises(exceptions.HyperVvSwitchNotFound,
                           self.netutils._get_vswitch,
                           self._FAKE_VSWITCH_NAME)
 
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_prepare_profile_sd')
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_profile_setting_data_from_port_alloc')
+    def _test_set_vswitch_port_profile_id(
+            self, mock_get_profile_setting_data_from_port_alloc,
+            mock_prepare_profile_sd, found, side_effect=None):
+        mock_port_profile = mock.MagicMock()
+        mock_new_port_profile = mock.MagicMock()
+        mock_port_alloc = self._mock_get_switch_port_alloc()
+
+        mock_add_feature = self.netutils._jobutils.add_virt_feature
+        mock_remove_feature = self.netutils._jobutils.remove_virt_feature
+
+        mock_get_profile_setting_data_from_port_alloc.return_value = (
+            mock_port_profile if found else None
+        )
+        mock_prepare_profile_sd.return_value = mock_new_port_profile
+        mock_add_feature.side_effect = side_effect
+
+        fake_params = {
+            "switch_port_name": self._FAKE_PORT_NAME,
+            "profile_id": mock.sentinel.profile_id,
+            "profile_data": mock.sentinel.profile_data,
+            "profile_name": mock.sentinel.profile_name,
+            "net_cfg_instance_id": None,
+            "cdn_label_id": None,
+            "cdn_label_string": None,
+            "vendor_id": None,
+            "vendor_name": mock.sentinel.vendor_name,
+        }
+
+        if side_effect:
+            self.assertRaises(
+                exceptions.HyperVException,
+                self.netutils.set_vswitch_port_profile_id,
+                **fake_params)
+        else:
+            self.netutils.set_vswitch_port_profile_id(**fake_params)
+
+        fake_params.pop("switch_port_name")
+        mock_prepare_profile_sd.assert_called_once_with(**fake_params)
+
+        if found:
+            mock_remove_feature.assert_called_once_with(mock_port_profile)
+            self.assertNotIn(self._FAKE_INSTANCE_ID,
+                             self.netutils._profile_sds)
+
+        mock_get_profile_setting_data_from_port_alloc.assert_called_with(
+            mock_port_alloc)
+
+        self.assertNotIn(mock_port_alloc, self.netutils._profile_sds)
+        mock_add_feature.assert_called_once_with(mock_new_port_profile,
+                                                 mock_port_alloc)
+
+    def test_set_vswitch_port_profile_id(self):
+        self._test_set_vswitch_port_profile_id(found=True)
+
+    def test_set_vswitch_port_profile_id_not_found(self):
+        self._test_set_vswitch_port_profile_id(found=False)
+
+    def test_set_vswitch_port_profile_id_failed(self):
+        self._test_set_vswitch_port_profile_id(found=False,
+                                               side_effect=Exception)
+
     def test_set_vswitch_port_vlan_id_invalid_mode(self):
         self.assertRaises(
             AttributeError, self.netutils.set_vswitch_port_vlan_id,
@@ -499,6 +648,47 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
 
         self.assertFalse(self.netutils._jobutils.add_virt_feature.called)
 
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_hw_offload_sd_from_port_alloc')
+    def test_set_vswitch_port_sriov_already_set(self, mock_get_hw_offload_sd):
+        mock_port_alloc = self._mock_get_switch_port_alloc()
+        mock_hw_offload_sd = mock_get_hw_offload_sd.return_value
+        mock_hw_offload_sd.IOVOffloadWeight = self.netutils._IOV_ENABLED
+
+        self.netutils.set_vswitch_port_sriov(mock.sentinel.port_name,
+                                             True)
+
+        mock_get_hw_offload_sd.assert_called_once_with(mock_port_alloc)
+        self.netutils._jobutils.modify_virt_feature.assert_not_called()
+
+    @ddt.data(True, False)
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_hw_offload_sd_from_port_alloc')
+    def test_set_vswitch_port_sriov(self, state, mock_get_hw_offload_sd):
+        mock_port_alloc = self._mock_get_switch_port_alloc()
+        mock_hw_offload_sd = mock_get_hw_offload_sd.return_value
+
+        self.netutils.set_vswitch_port_sriov(mock.sentinel.port_name,
+                                             state)
+
+        mock_get_hw_offload_sd.assert_called_once_with(mock_port_alloc)
+        self.netutils._jobutils.modify_virt_feature.assert_called_with(
+            mock_hw_offload_sd)
+        desired_state = (self.netutils._IOV_ENABLED if state else
+                         self.netutils._IOV_DISABLED)
+        self.assertEqual(desired_state, mock_hw_offload_sd.IOVOffloadWeight)
+
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_setting_data_from_port_alloc')
+    def test_get_profile_setting_data_from_port_alloc(self, mock_get_sd):
+        result = self.netutils._get_profile_setting_data_from_port_alloc(
+            mock.sentinel.port)
+
+        self.assertEqual(mock_get_sd.return_value, result)
+        mock_get_sd.assert_called_once_with(
+            mock.sentinel.port, self.netutils._profile_sds,
+            self.netutils._PORT_PROFILE_SET_DATA)
+
     @mock.patch.object(networkutils.NetworkUtils,
                        '_get_setting_data_from_port_alloc')
     def test_get_vlan_setting_data_from_port_alloc(self, mock_get_sd):
@@ -522,6 +712,17 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
             mock_port, self.netutils._vsid_sds,
             self.netutils._PORT_SECURITY_SET_DATA)
 
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_setting_data_from_port_alloc')
+    def test_get_hw_offload_sd_from_port_alloc(self, mock_get_sd):
+        mock_port = mock.MagicMock()
+        result = self.netutils._get_hw_offload_sd_from_port_alloc(mock_port)
+
+        self.assertEqual(mock_get_sd.return_value, result)
+        mock_get_sd.assert_called_once_with(
+            mock_port, self.netutils._hw_offload_sds,
+            self.netutils._PORT_HW_OFFLOAD_SET_DATA)
+
     @mock.patch.object(networkutils.NetworkUtils,
                        '_get_setting_data_from_port_alloc')
     def test_get_bandwidth_setting_data_from_port_alloc(self, mock_get_sd):
@@ -543,8 +744,11 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
 
         self.assertEqual(mock.sentinel.sd_object, result)
 
+    @ddt.data(True, False)
     @mock.patch.object(_wqlutils, 'get_element_associated_class')
-    def test_get_setting_data_from_port_alloc(self, mock_get_elem_assoc_cls):
+    def test_get_setting_data_from_port_alloc(self, enable_cache,
+                                              mock_get_elem_assoc_cls):
+        self.netutils._enable_cache = enable_cache
         sd_object = mock.MagicMock()
         mock_port = mock.MagicMock(InstanceID=mock.sentinel.InstanceID)
         mock_get_elem_assoc_cls.return_value = [sd_object]
@@ -556,7 +760,10 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
             self.netutils._conn, mock.sentinel.data_class,
             element_instance_id=mock.sentinel.InstanceID)
         self.assertEqual(sd_object, result)
-        self.assertEqual(sd_object, cache[mock.sentinel.InstanceID])
+
+        expected_cache = ({mock.sentinel.InstanceID: sd_object}
+                          if enable_cache else {})
+        self.assertEqual(expected_cache, cache)
 
     def test_get_switch_port_allocation_cached(self):
         self.netutils._switch_ports[mock.sentinel.port_name] = (
@@ -568,8 +775,11 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
         self.assertEqual(mock.sentinel.port, port)
         self.assertTrue(found)
 
+    @ddt.data(True, False)
     @mock.patch.object(networkutils.NetworkUtils, '_get_setting_data')
-    def test_get_switch_port_allocation(self, mock_get_set_data):
+    def test_get_switch_port_allocation(self, enable_cache, mock_get_set_data):
+        self.netutils._enable_cache = enable_cache
+        self.netutils._switch_ports = {}
         mock_get_set_data.return_value = (mock.sentinel.port, True)
 
         port, found = self.netutils._get_switch_port_allocation(
@@ -577,7 +787,9 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
 
         self.assertEqual(mock.sentinel.port, port)
         self.assertTrue(found)
-        self.assertIn(mock.sentinel.port_name, self.netutils._switch_ports)
+        expected_cache = ({mock.sentinel.port_name: port}
+                          if enable_cache else {})
+        self.assertEqual(expected_cache, self.netutils._switch_ports)
         mock_get_set_data.assert_called_once_with(
             self.netutils._PORT_ALLOC_SET_DATA, mock.sentinel.port_name, False)
 
@@ -750,8 +962,11 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
 
         self.assertEqual([mock.sentinel.fake_acl], acls)
 
+    @ddt.data(True, False)
     @mock.patch.object(_wqlutils, 'get_element_associated_class')
-    def test_get_port_security_acls(self, mock_get_elem_assoc_cls):
+    def test_get_port_security_acls(self, enable_cache,
+                                    mock_get_elem_assoc_cls):
+        self.netutils._enable_cache = enable_cache
         self.netutils._sg_acl_sds = {}
         mock_port = mock.MagicMock()
         mock_get_elem_assoc_cls.return_value = [mock.sentinel.fake_acl]
@@ -759,7 +974,9 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
         acls = self.netutils._get_port_security_acls(mock_port)
 
         self.assertEqual([mock.sentinel.fake_acl], acls)
-        self.assertEqual({mock_port.ElementName: [mock.sentinel.fake_acl]},
+        expected_cache = ({mock_port.ElementName: [mock.sentinel.fake_acl]}
+                          if enable_cache else {})
+        self.assertEqual(expected_cache,
                          self.netutils._sg_acl_sds)
         mock_get_elem_assoc_cls.assert_called_once_with(
             self.netutils._conn, self.netutils._PORT_EXT_ACL_SET_DATA,
@@ -916,6 +1133,47 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
         mock_remove_feature.assert_called_once_with(
             mock_bandwidth_settings)
 
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_create_default_setting_data')
+    def test_prepare_profile_sd(self, mock_create_default_sd):
+        mock_profile_sd = mock_create_default_sd.return_value
+
+        actual_profile_sd = self.netutils._prepare_profile_sd(
+            profile_id=mock.sentinel.profile_id,
+            profile_data=mock.sentinel.profile_data,
+            profile_name=mock.sentinel.profile_name,
+            net_cfg_instance_id=mock.sentinel.net_cfg_instance_id,
+            cdn_label_id=mock.sentinel.cdn_label_id,
+            cdn_label_string=mock.sentinel.cdn_label_string,
+            vendor_id=mock.sentinel.vendor_id,
+            vendor_name=mock.sentinel.vendor_name)
+
+        self.assertEqual(mock_profile_sd, actual_profile_sd)
+        self.assertEqual(mock.sentinel.profile_id,
+                         mock_profile_sd.ProfileId)
+        self.assertEqual(mock.sentinel.profile_data,
+                         mock_profile_sd.ProfileData)
+        self.assertEqual(mock.sentinel.profile_name,
+                         mock_profile_sd.ProfileName)
+        self.assertEqual(mock.sentinel.net_cfg_instance_id,
+                         mock_profile_sd.NetCfgInstanceId)
+        self.assertEqual(mock.sentinel.cdn_label_id,
+                         mock_profile_sd.CdnLabelId)
+        self.assertEqual(mock.sentinel.cdn_label_string,
+                         mock_profile_sd.CdnLabelString)
+        self.assertEqual(mock.sentinel.vendor_id,
+                         mock_profile_sd.VendorId)
+        self.assertEqual(mock.sentinel.vendor_name,
+                         mock_profile_sd.VendorName)
+        mock_create_default_sd.assert_called_once_with(
+            self.netutils._PORT_PROFILE_SET_DATA)
+
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_create_default_setting_data')
+    def test_prepare_profile_sd_failed(self, mock_create_default_sd):
+        self.assertRaises(TypeError, self.netutils._prepare_profile_sd,
+                          invalid_argument=mock.sentinel.invalid_argument)
+
 
 class TestNetworkUtilsR2(test_base.OsWinBaseTestCase):
 
