@@ -179,6 +179,21 @@ class DiskUtilsTestCase(test_base.OsWinBaseTestCase):
 
         mock_rescan.assert_called_once_with()
 
+    @mock.patch.object(diskutils, '_RESCAN_LOCK')
+    @mock.patch.object(diskutils.DiskUtils, '_rescan_disks')
+    def test_rescan_merge_requests(self, mock_rescan_helper, mock_rescan_lock):
+        mock_rescan_lock.locked.side_effect = [False, True, True]
+
+        self._diskutils.rescan_disks(merge_requests=True)
+        self._diskutils.rescan_disks(merge_requests=True)
+        self._diskutils.rescan_disks(merge_requests=False)
+
+        exp_rescan_count = 2
+        mock_rescan_helper.assert_has_calls(
+            [mock.call()] * exp_rescan_count)
+        mock_rescan_lock.__enter__.assert_has_calls(
+            [mock.call()] * exp_rescan_count)
+
     @mock.patch('time.sleep')
     def test_rescan_disks_error(self, mock_sleep):
         mock_rescan = self._get_mocked_wmi_rescan(return_value=1)
@@ -241,6 +256,15 @@ class DiskUtilsTestCase(test_base.OsWinBaseTestCase):
         self._test_get_disk_capacity(
             raised_exc=exceptions.Win32Exception)
 
+    @mock.patch.object(diskutils.DiskUtils, '_get_disk_by_number')
+    def test_get_disk_size(self, mock_get_disk):
+        disk_size = self._diskutils.get_disk_size(
+            mock.sentinel.disk_number)
+
+        self.assertEqual(mock_get_disk.return_value.Size, disk_size)
+
+        mock_get_disk.assert_called_once_with(mock.sentinel.disk_number)
+
     def test_parse_scsi_id_desc(self):
         vpd_str = ('008300240103001060002AC00000000000000EA0'
                    '0000869902140004746573740115000400000001')
@@ -354,3 +378,72 @@ class DiskUtilsTestCase(test_base.OsWinBaseTestCase):
         result = self._diskutils._select_supported_scsi_identifiers(
             identifiers)
         self.assertEqual(expected_identifiers, result)
+
+    def test_get_new_disk_policy(self):
+        mock_setting_obj = mock.Mock()
+        setting_cls = self._diskutils._conn_storage.MSFT_StorageSetting
+        setting_cls.Get.return_value = (0, mock_setting_obj)
+
+        policy = self._diskutils.get_new_disk_policy()
+        self.assertEqual(mock_setting_obj.NewDiskPolicy, policy)
+
+    def test_set_new_disk_policy(self):
+        self._diskutils.set_new_disk_policy(mock.sentinel.policy)
+
+        setting_cls = self._diskutils._conn_storage.MSFT_StorageSetting
+        setting_cls.Set.assert_called_once_with(
+            NewDiskPolicy=mock.sentinel.policy)
+
+    @mock.patch.object(diskutils.DiskUtils, '_get_disk_by_number')
+    @ddt.data(0, 1)
+    def test_set_disk_online(self, err_code, mock_get_disk):
+        mock_disk = mock_get_disk.return_value
+        mock_disk.Online.return_value = (mock.sentinel.ext_err_info,
+                                         err_code)
+
+        if err_code:
+            self.assertRaises(exceptions.DiskUpdateError,
+                              self._diskutils.set_disk_online,
+                              mock.sentinel.disk_number)
+        else:
+            self._diskutils.set_disk_online(mock.sentinel.disk_number)
+
+        mock_disk.Online.assert_called_once_with()
+        mock_get_disk.assert_called_once_with(mock.sentinel.disk_number)
+
+    @mock.patch.object(diskutils.DiskUtils, '_get_disk_by_number')
+    @ddt.data(0, 1)
+    def test_set_disk_offline(self, err_code, mock_get_disk):
+        mock_disk = mock_get_disk.return_value
+        mock_disk.Offline.return_value = (mock.sentinel.ext_err_info,
+                                          err_code)
+
+        if err_code:
+            self.assertRaises(exceptions.DiskUpdateError,
+                              self._diskutils.set_disk_offline,
+                              mock.sentinel.disk_number)
+        else:
+            self._diskutils.set_disk_offline(mock.sentinel.disk_number)
+
+        mock_disk.Offline.assert_called_once_with()
+        mock_get_disk.assert_called_once_with(mock.sentinel.disk_number)
+
+    @mock.patch.object(diskutils.DiskUtils, '_get_disk_by_number')
+    @ddt.data(0, 1)
+    def test_set_disk_readonly(self, err_code, mock_get_disk):
+        mock_disk = mock_get_disk.return_value
+        mock_disk.SetAttributes.return_value = (mock.sentinel.ext_err_info,
+                                                err_code)
+
+        if err_code:
+            self.assertRaises(exceptions.DiskUpdateError,
+                              self._diskutils.set_disk_readonly_status,
+                              mock.sentinel.disk_number,
+                              read_only=True)
+        else:
+            self._diskutils.set_disk_readonly_status(
+                mock.sentinel.disk_number,
+                read_only=True)
+
+        mock_disk.SetAttributes.assert_called_once_with(IsReadOnly=True)
+        mock_get_disk.assert_called_once_with(mock.sentinel.disk_number)
