@@ -22,6 +22,7 @@ from os_win import _utils
 from os_win import exceptions
 from os_win.utils import win32utils
 from os_win.utils.winapi import constants as w_const
+from os_win.utils.winapi import wintypes
 
 
 @ddt.ddt
@@ -46,7 +47,6 @@ class Win32UtilsTestCase(base.BaseTestCase):
 
         mock.patch.multiple(win32utils,
                             kernel32=mock.DEFAULT,
-                            wintypes=mock.DEFAULT,
                             create=True).start()
 
     @mock.patch.object(win32utils.Win32Utils, 'get_error_message')
@@ -208,3 +208,36 @@ class Win32UtilsTestCase(base.BaseTestCase):
 
         mock_localfree.assert_any_call(mock.sentinel.handle)
         self.assertEqual(bool(ret_val), mock_log_exc.called)
+
+    @mock.patch.object(win32utils.Win32Utils, 'run_and_check_output')
+    def test_wait_for_multiple_objects(self, mock_helper):
+        fake_handles = [10, 11]
+
+        ret_val = self._win32_utils.wait_for_multiple_objects(
+            fake_handles, mock.sentinel.wait_all, mock.sentinel.milliseconds)
+
+        mock_helper.assert_called_once_with(
+            win32utils.kernel32.WaitForMultipleObjects,
+            len(fake_handles),
+            mock.ANY,
+            mock.sentinel.wait_all,
+            mock.sentinel.milliseconds,
+            kernel32_lib_func=True,
+            error_ret_vals=[w_const.WAIT_FAILED])
+        self.assertEqual(mock_helper.return_value, ret_val)
+
+        handles_arg = mock_helper.call_args_list[0][0][2]
+        self.assertIsInstance(handles_arg,
+                              wintypes.HANDLE * len(fake_handles))
+        self.assertEqual(fake_handles, handles_arg[:])
+
+    @mock.patch.object(win32utils.Win32Utils, 'run_and_check_output')
+    def test_wait_for_multiple_objects_timeout(self, mock_helper):
+        fake_handles = [10]
+        mock_helper.return_value = w_const.ERROR_WAIT_TIMEOUT
+
+        self.assertRaises(
+            exceptions.Timeout,
+            self._win32_utils.wait_for_multiple_objects,
+            fake_handles, mock.sentinel.wait_all,
+            mock.sentinel.milliseconds)
