@@ -113,8 +113,12 @@ utils_map = {
     'vmutils': {
         'VMUtils': {
             'min_version': 6.2,
-            'max_version': 10,
+            'max_version': 6.3,
             'path': 'os_win.utils.compute.vmutils.VMUtils'},
+        'VMUtils6_3': {
+            'min_version': 6.3,
+            'max_version': 10,
+            'path': 'os_win.utils.compute.vmutils.VMUtils6_3'},
         'VMUtils10': {
             'min_version': 10,
             'max_version': None,
@@ -128,7 +132,12 @@ utils_map = {
         'DNSUtils': {
             'min_version': 6.2,
             'max_version': None,
-            'path': 'os_win.utils.dns.dnsutils.DNSUtils'}}
+            'path': 'os_win.utils.dns.dnsutils.DNSUtils'}},
+    'processutils': {
+        'ProcessUtils': {
+            'min_version': 6.2,
+            'max_version': None,
+            'path': 'os_win.utils.processutils.ProcessUtils'}}
 }
 
 
@@ -226,3 +235,7 @@ def get_dnsutils():
 
 def get_migrationutils():
     return _get_class(class_type='migrationutils')
+
+
+def get_processutils():
+    return _get_class(class_type='processutils')
